@@ -1,7 +1,6 @@
 pkgname <- "pROC"
 source(file.path(R.home("share"), "R", "examples-header.R"))
 options(warn = 1)
-options(pager = "console")
 library('pROC')
 
 assign(".oldSearch", search(), pos = 'CheckExEnv')
@@ -368,6 +367,10 @@ coords(rocobj, 0.5, "se", ret="se")
 # fully qualified but identical:
 coords(roc=rocobj, x=0.5, input="sensitivity", ret="sensitivity")
 
+# Compare with drop=FALSE
+coords(rocobj, 0.55, drop=FALSE)
+coords(rocobj, 0.9, "specificity", as.list=TRUE, drop=FALSE)
+
 # Same in percent
 rocobj <- roc(aSAH$outcome, aSAH$s100b, percent=TRUE)
 
@@ -375,6 +378,12 @@ coords(rocobj, 0.55)
 coords(rocobj, 90, "specificity", as.list=TRUE)
 coords(rocobj, x=50, input="sensitivity", ret=c("sen", "spec"))
 
+# And with a smoothed ROC curve
+coords(smooth(rocobj), 90, "specificity")
+coords(smooth(rocobj), 90, "specificity", drop=FALSE)
+coords(smooth(rocobj), 90, "specificity", as.list=TRUE)
+coords(smooth(rocobj), 90, "specificity", as.list=TRUE, drop=FALSE)
+
 # Get the sensitivities for all thresholds
 sensitivities <- coords(rocobj, rocobj$thresholds, "thr", "se")
 # This is equivalent to taking sensitivities from rocobj directly
@@ -401,6 +410,128 @@ plot(rocobj, print.thres="best", print.thres.best.method="youden",
 plot(rocobj, print.thres="best", print.thres.best.method="closest.topleft",
                                  print.thres.best.weights=c(5, 0.2)) 
 
+# Return more values:
+coords(rocobj, "best", ret=c("threshold", "specificity", "sensitivity", "accuracy",
+                           "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity",
+                           "1-sensitivity", "1-npv", "1-ppv"))
+coords(smooth(rocobj), "best", ret=c("threshold", "specificity", "sensitivity", "accuracy",
+                           "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity",
+                           "1-sensitivity", "1-npv", "1-ppv"))
+coords(smooth(rocobj), 0.5, ret=c("threshold", "specificity", "sensitivity", "accuracy",
+                           "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity",
+                           "1-sensitivity", "1-npv", "1-ppv"))
+
+
+
+cleanEx()
+nameEx("cov")
+### * cov
+
+flush(stderr()); flush(stdout())
+
+### Name: cov.roc
+### Title: Covariance of two paired ROC curves
+### Aliases: cov cov.default cov.auc cov.smooth.roc cov.roc
+### Keywords: multivariate nonparametric utilities roc
+
+### ** Examples
+
+data(aSAH)
+
+# Basic example with 2 roc objects
+roc1 <- roc(aSAH$outcome, aSAH$s100b)
+roc2 <- roc(aSAH$outcome, aSAH$wfns)
+cov(roc1, roc2)
+
+## Not run: 
+##D # The latter used Delong. To use bootstrap:
+##D cov(roc1, roc2, method="bootstrap")
+##D # Decrease boot.n for a faster execution:
+##D cov(roc1, roc2, method="bootstrap", boot.n=1000)
+## End(Not run)
+
+## Not run: 
+##D # Comparison can be done on smoothed ROCs
+##D # Smoothing is re-done at each iteration, and execution is slow
+##D cov(smooth(roc1), smooth(roc2))
+## End(Not run)
+# or from an AUC (no smoothing)
+cov(auc(roc1), roc2)
+
+## Not run: 
+##D # With bootstrap and return.values, one can compute the variances of the
+##D # ROC curves in one single bootstrap run:
+##D cov.rocs <- cov(roc1, roc2, method="bootstrap", boot.return=TRUE)
+##D # var(roc1):
+##D var(attr(cov.rocs, "resampled.values")[,1])
+##D # var(roc2):
+##D var(attr(cov.rocs, "resampled.values")[,2])
+## End(Not run)
+
+## Not run: 
+##D # Covariance of partial AUC:
+##D roc3 <- roc(aSAH$outcome, aSAH$s100b, partial.auc=c(1, 0.8), partial.auc.focus="se")
+##D roc4 <- roc(aSAH$outcome, aSAH$wfns, partial.auc=c(1, 0.8), partial.auc.focus="se")
+##D cov(roc3, roc4)
+##D # This is strictly equivalent to:
+##D cov(roc3, roc4, method="bootstrap")
+##D 
+##D # Alternatively, we could re-use roc1 and roc2 to get the same result:
+##D cov(roc1, roc2, reuse.auc=FALSE, partial.auc=c(1, 0.8), partial.auc.focus="se")
+## End(Not run)
+
+# Spurious use of DeLong's test with different direction:
+roc5 <- roc(aSAH$outcome, aSAH$s100b, direction="<")
+roc6 <- roc(aSAH$outcome, aSAH$s100b, direction=">")
+cov(roc5, roc6, method="delong")
+
+## Test data from Hanley and Hajian-Tilaki, 1997
+disease.present <- c("Yes", "No", "Yes", "No", "No", "Yes", "Yes", "No", "No", "Yes", "No", "No", "Yes", "No", "No")
+field.strength.1 <- c(1, 2, 5, 1, 1, 1, 2, 1, 2, 2, 1, 1, 5, 1, 1)
+field.strength.2 <- c(1, 1, 5, 1, 1, 1, 4, 1, 2, 2, 1, 1, 5, 1, 1)
+roc7 <- roc(disease.present, field.strength.1)
+roc8 <- roc(disease.present, field.strength.2)
+# Assess the covariance:
+cov(roc7, roc8)
+
+## Not run: 
+##D # With bootstrap:
+##D cov(roc7, roc8, method="bootstrap")
+## End(Not run)
+
+
+
+
+cleanEx()
+nameEx("has.partial.auc")
+### * has.partial.auc
+
+flush(stderr()); flush(stdout())
+
+### Name: has.partial.auc
+### Title: Does the ROC curve have a partial AUC?
+### Aliases: has.partial.auc has.partial.auc.smooth.roc has.partial.auc.auc
+###   has.partial.auc.roc
+### Keywords: programming logic roc
+
+### ** Examples
+
+data(aSAH)
+
+# Full AUC
+roc1 <- roc(aSAH$outcome, aSAH$s100b)
+has.partial.auc(roc1)
+has.partial.auc(auc(roc1))
+has.partial.auc(smooth(roc1))
+
+# Partial AUC
+roc2 <- roc(aSAH$outcome, aSAH$s100b, partial.auc = c(1, 0.9))
+has.partial.auc(roc2)
+has.partial.auc(smooth(roc2))
+
+# No AUC
+roc3 <- roc(aSAH$outcome, aSAH$s100b, auc = FALSE)
+has.partial.auc(roc3)
 
 
 
@@ -498,6 +629,10 @@ roc1 <- roc(aSAH$outcome,
 roc2 <- roc(aSAH$outcome, aSAH$wfns,
             plot=TRUE, add=TRUE, percent=roc1$percent)
 
+## Coordinates of the curve ##
+coords(roc1, "best", ret=c("threshold", "specificity", "1-npv"))
+coords(roc2, "local maximas", ret=c("threshold", "sens", "spec", "ppv", "npv"))
+
 ## Confidence intervals ##
 
 # CI of the AUC
@@ -728,7 +863,7 @@ nameEx("roc.test")
 flush(stderr()); flush(stdout())
 
 ### Name: roc.test
-### Title: Compare the AUC of two correlated ROC curves
+### Title: Compare the AUC of two ROC curves
 ### Aliases: roc.test roc.test.default roc.test.roc roc.test.formula
 ###   roc.test.auc roc.test.smooth.roc
 ### Keywords: multivariate nonparametric utilities htest roc
@@ -763,8 +898,7 @@ roc.test(roc1, roc2, alternative="less")
 ##D # Smoothing is re-done at each iteration, and execution is slow
 ##D roc.test(smooth(roc1), smooth(roc2))
 ##D # or:
-##D roc.test(aSAH$outcome, aSAH$s100b, aSAH$wfns, smooth=TRUE,
-##D smooth.method="density", boot.n=100)
+##D roc.test(aSAH$outcome, aSAH$s100b, aSAH$wfns, smooth=TRUE, boot.n=100)
 ## End(Not run)
 # or from an AUC (no smoothing)
 roc.test(auc(roc1), roc2)
@@ -837,8 +971,13 @@ rs2 <- smooth(rocobj, method="density")
 plot(rs2, add=TRUE, col="blue")
 rs3 <- smooth(rocobj, method="fitdistr", density="lognormal")
 plot(rs3, add=TRUE, col="magenta")
-legend("bottomright", legend=c("Empirical", "Binormal", "Density", "Log-normal"),
-       col=c("black", "green", "blue", "magenta"), lwd=2)
+rs4 <- smooth(rocobj, method="logcondens")
+plot(rs4, add=TRUE, col="brown")
+rs5 <- smooth(rocobj, method="logcondens.smooth")
+plot(rs5, add=TRUE, col="orange")
+legend("bottomright", legend=c("Empirical", "Binormal", "Density", "Log-normal",
+                               "Log-concave density", "Smoothed log-concave density"),
+       col=c("black", "green", "blue", "magenta", "brown", "orange"), lwd=2)
 
 ## Advanced smoothing
 
@@ -858,45 +997,59 @@ smooth(rocobj, method="density", density.controls=density.controls$y,
 # which is roughly what is done by a simple:
 smooth(rocobj, method="density")
 
+# With logcondens:
+smooth(rocobj, method="logcondens")
+smooth(rocobj, method="logcondens.smooth")
 
-## Smoothing artificial ROC curves
-
-rand.unif <- runif(1000, -1, 1)
-rand.exp <- rexp(1000)
-rand.norm <- 
-rnorm(1000)
-
-# two normals
-roc.norm <- roc(controls=rnorm(1000), cases=rnorm(1000)+1, plot=TRUE)
-plot(smooth(roc.norm), col="green", lwd=1, add=TRUE)
-plot(smooth(roc.norm, method="density"), col="red", lwd=1, add=TRUE)
-plot(smooth(roc.norm, method="fitdistr"), col="blue", lwd=1, add=TRUE)
-legend("bottomright", legend=c("empirical", "binormal", "density", "fitdistr"),
-       col=c(par("fg"), "green", "red", "blue"), lwd=c(2, 1, 1, 1))
-       
-# deviation from the normality
-roc.norm.exp <- roc(controls=rnorm(1000), cases=rexp(1000), plot=TRUE)
-plot(smooth(roc.norm.exp), col="green", lwd=1, add=TRUE)
-plot(smooth(roc.norm.exp, method="density"), col="red", lwd=1, add=TRUE)
-# Wrong fitdistr: normality assumed by default
-plot(smooth(roc.norm.exp, method="fitdistr"), col="blue", lwd=1, add=TRUE)
-# Correct fitdistr
-plot(smooth(roc.norm.exp, method="fitdistr", density.controls="normal",
-            density.cases="exponential"), col="purple", lwd=1, add=TRUE)
-legend("bottomright", legend=c("empirical", "binormal", "density",
-                               "wrong fitdistr", "correct fitdistr"),
-       col=c(par("fg"), "green", "red", "blue", "purple"), lwd=c(2, 1, 1, 1, 1))
-
-# large deviation from the normality
-roc.unif.exp <- roc(controls=runif(1000, 2, 3), cases=rexp(1000)+2, plot=TRUE)
-plot(smooth(roc.unif.exp), col="green", lwd=1, add=TRUE)
-plot(smooth(roc.unif.exp, method="density"), col="red", lwd=1, add=TRUE)
-plot(smooth(roc.unif.exp, method="density", bw="ucv"), col="magenta", lwd=1, add=TRUE)
-# Wrong fitdistr: normality assumed by default (uniform distributions not handled)
-plot(smooth(roc.unif.exp, method="fitdistr"), col="blue", lwd=1, add=TRUE)
-legend("bottomright", legend=c("empirical", "binormal", "density",
-                               "density ucv", "wrong fitdistr"),
-       col=c(par("fg"), "green", "red", "magenta", "blue"), lwd=c(2, 1, 1, 1, 1))
+## Not run: 
+##D ## Smoothing artificial ROC curves
+##D 
+##D rand.unif <- runif(1000, -1, 1)
+##D rand.exp <- rexp(1000)
+##D rand.norm <- 
+##D rnorm(1000)
+##D 
+##D # two normals
+##D roc.norm <- roc(controls=rnorm(1000), cases=rnorm(1000)+1, plot=TRUE)
+##D plot(smooth(roc.norm), col="green", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm, method="density"), col="red", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm, method="fitdistr"), col="blue", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm, method="logcondens"), col="brown", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm, method="logcondens.smooth"), col="orange", lwd=1, add=TRUE)
+##D legend("bottomright", legend=c("empirical", "binormal", "density", "fitdistr",
+##D                                "logcondens", "logcondens.smooth"), 
+##D        col=c(par("fg"), "green", "red", "blue", "brown", "orange"), lwd=c(2, 1, 1, 1))
+##D        
+##D # deviation from the normality
+##D roc.norm.exp <- roc(controls=rnorm(1000), cases=rexp(1000), plot=TRUE)
+##D plot(smooth(roc.norm.exp), col="green", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm.exp, method="density"), col="red", lwd=1, add=TRUE)
+##D # Wrong fitdistr: normality assumed by default
+##D plot(smooth(roc.norm.exp, method="fitdistr"), col="blue", lwd=1, add=TRUE)
+##D # Correct fitdistr
+##D plot(smooth(roc.norm.exp, method="fitdistr", density.controls="normal",
+##D             density.cases="exponential"), col="purple", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm.exp, method="logcondens"), col="brown", lwd=1, add=TRUE)
+##D plot(smooth(roc.norm.exp, method="logcondens.smooth"), col="orange", lwd=1, add=TRUE)
+##D legend("bottomright", legend=c("empirical", "binormal", "density",
+##D                                "wrong fitdistr", "correct fitdistr",
+##D                                "logcondens", "logcondens.smooth"),
+##D        col=c(par("fg"), "green", "red", "blue", "purple", "brown", "orange"), lwd=c(2, 1, 1, 1, 1))
+##D 
+##D # large deviation from the normality
+##D roc.unif.exp <- roc(controls=runif(1000, 2, 3), cases=rexp(1000)+2, plot=TRUE)
+##D plot(smooth(roc.unif.exp), col="green", lwd=1, add=TRUE)
+##D plot(smooth(roc.unif.exp, method="density"), col="red", lwd=1, add=TRUE)
+##D plot(smooth(roc.unif.exp, method="density", bw="ucv"), col="magenta", lwd=1, add=TRUE)
+##D # Wrong fitdistr: normality assumed by default (uniform distributions not handled)
+##D plot(smooth(roc.unif.exp, method="fitdistr"), col="blue", lwd=1, add=TRUE)
+##D plot(smooth(roc.unif.exp, method="logcondens"), col="brown", lwd=1, add=TRUE)
+##D plot(smooth(roc.unif.exp, method="logcondens.smooth"), col="orange", lwd=1, add=TRUE)
+##D legend("bottomright", legend=c("empirical", "binormal", "density",
+##D                                "density ucv", "wrong fitdistr",
+##D                                "logcondens", "logcondens.smooth"),
+##D        col=c(par("fg"), "green", "red", "magenta", "blue", "brown", "orange"), lwd=c(2, 1, 1, 1, 1))
+## End(Not run)
 
 # 2 uniform distributions with a custom density function
 unif.density <- function(x, n, from, to, bw, kernel, ...) {
@@ -916,7 +1069,65 @@ plot(s, add=TRUE, col="grey")
 
 
 
-graphics::par(get("par.postscript", pos = 'CheckExEnv'))
+cleanEx()
+nameEx("var")
+### * var
+
+flush(stderr()); flush(stdout())
+
+### Name: var.roc
+### Title: Variance of a ROC curve
+### Aliases: var var.roc var.auc var.smooth.roc var.default
+### Keywords: univar nonparametric utilities roc
+
+### ** Examples
+
+data(aSAH)
+
+##  Basic example
+roc1 <- roc(aSAH$outcome, aSAH$s100b)
+roc2 <- roc(aSAH$outcome, aSAH$wfns)
+var(roc1)
+var(roc2)
+
+# We could also write it in one line:
+var(roc(aSAH$outcome, aSAH$s100b))
+
+## Not run: 
+##D # The latter used Delong. To use bootstrap:
+##D var(roc1, method="bootstrap")
+##D # Decrease boot.n for a faster execution
+##D var(roc1,method="bootstrap", boot.n=1000)
+## End(Not run)
+
+## Not run: 
+##D # Variance of smoothed ROCs:
+##D # Smoothing is re-done at each iteration, and execution is slow
+##D var(smooth(roc1))
+## End(Not run)
+
+# or from an AUC (no smoothing)
+var(auc(roc1))
+
+## Test data from Hanley and Hajian-Tilaki, 1997
+disease.present <- c("Yes", "No", "Yes", "No", "No", "Yes", "Yes", "No", "No", "Yes", "No", "No", "Yes", "No", "No")
+field.strength.1 <- c(1, 2, 5, 1, 1, 1, 2, 1, 2, 2, 1, 1, 5, 1, 1)
+field.strength.2 <- c(1, 1, 5, 1, 1, 1, 4, 1, 2, 2, 1, 1, 5, 1, 1)
+roc3 <- roc(disease.present, field.strength.1)
+roc4 <- roc(disease.present, field.strength.2)
+# Assess the variance:
+var(roc3)
+var(roc4)
+
+## Not run: 
+##D # With bootstrap:
+##D var(roc3, method="bootstrap")
+##D var(roc4, method="bootstrap")
+## End(Not run)
+
+
+
+
 ### * <FOOTER>
 ###
 cat("Time elapsed: ", proc.time() - get("ptime", pos = 'CheckExEnv'),"\n")
